
USE AdventureWorks
GO

-- Find root
SELECT * 
FROM EmployeeAdj
WHERE ManagerID IS NULL;

-- Find leaves
SELECT *
FROM EmployeeAdj AS Boss
WHERE NOT EXISTS 
 (SELECT * FROM EmployeeAdj AS Worker
  WHERE Worker.ManagerID = Boss.EmployeeID);
GO  
  
-- All subordinates for one boss, any level
DECLARE @BossID INT; -- use for parameterized version  
WITH Subtree(Boss, Worker)
AS
(
SELECT Worker.ManagerID, Worker.EmployeeID
FROM EmployeeAdj AS Worker
WHERE Worker.ManagerID = 3  -- @BossID
UNION ALL
SELECT Worker.ManagerID, Worker.EmployeeID
FROM EmployeeAdj AS Worker 
INNER JOIN
Subtree AS S ON S.Worker = Worker.ManagerID
) 
SELECT Worker FROM Subtree;
GO
  
-- Subtrees
WITH Subtree(Boss, Worker, OriginalWorker)
AS
(
SELECT Worker.ManagerID, Worker.EmployeeID, Worker.EmployeeID as OriginalWorker 
FROM EmployeeAdj AS Worker
UNION ALL
SELECT Worker.ManagerID, Worker.EmployeeID, S.OriginalWorker 
FROM EmployeeAdj AS Worker 
INNER JOIN
Subtree AS S ON S.Worker = Worker.ManagerID
) 
SELECT OriginalWorker, Worker FROM Subtree;
GO

-- or (not counting self)
WITH Subtree(Boss, Worker, OriginalWorker)
AS
(
SELECT Worker.ManagerID, Worker.EmployeeID, Worker.EmployeeID as OriginalWorker 
FROM EmployeeAdj AS Worker
UNION ALL
SELECT Worker.ManagerID, Worker.EmployeeID, S.OriginalWorker 
FROM EmployeeAdj AS Worker 
INNER JOIN
Subtree AS S ON S.Worker = Worker.ManagerID
) 
SELECT OriginalWorker, Worker FROM Subtree
WHERE OriginalWorker <> Worker;
GO

-- Find level of each Worker
WITH Subtree(Boss, Worker, Lvl)
AS
(
SELECT Worker.ManagerID, Worker.EmployeeID, Lvl = 0
FROM EmployeeAdj AS Worker
WHERE Worker.ManagerID IS NULL
UNION ALL
SELECT Worker.ManagerID, Worker.EmployeeID, Lvl = Lvl + 1
FROM EmployeeAdj AS Worker 
INNER JOIN
Subtree AS S ON S.Worker = Worker.ManagerID
) 
SELECT Worker, Lvl FROM Subtree
GO

-- Find height of a tree
WITH Subtree(Boss, Worker, Lvl)
AS
(
SELECT Worker.ManagerID, Worker.EmployeeID, Lvl = 0
FROM EmployeeAdj AS Worker
WHERE Worker.ManagerID IS NULL
UNION ALL
SELECT Worker.ManagerID, Worker.EmployeeID, Lvl = Lvl + 1
FROM EmployeeAdj AS Worker 
INNER JOIN
Subtree AS S ON S.Worker = Worker.ManagerID
) 
SELECT MAX(Lvl) FROM Subtree
GO

-- Levels Of Subordinates
-- Immediate Subordinates
SELECT ManagerID, EmployeeID
FROM EmployeeAdj
WHERE ManagerID IS NOT NULL
UNION ALL
SELECT EmployeeID, EmployeeID
FROM EmployeeAdj

-- not counting self
SELECT ManagerID, EmployeeID
FROM EmployeeAdj
WHERE ManagerID IS NOT NULL

-- Generalize to any level of hierarchy
DECLARE @n INT = 2;
DECLARE @my_member INT = 109;
WITH Subtree(Boss, Worker, Lvl)
AS
(
SELECT Worker.ManagerID, Worker.EmployeeID, Lvl = 1
FROM EmployeeAdj AS Worker
WHERE Worker.ManagerID = @my_member
UNION ALL
SELECT Worker.ManagerID, Worker.EmployeeID, Lvl = Lvl + 1
FROM EmployeeAdj AS Worker 
INNER JOIN
Subtree AS S ON S.Worker = Worker.ManagerID
) 
SELECT Subtree.Worker, 'is', @n, 'levels down from', @my_member FROM Subtree
WHERE Lvl = @n;
GO

-- all bosses for a specific employee
DECLARE @orig_worker INT = 114;
WITH Bosses(Boss, Worker, Lvl)
AS
(
SELECT Worker.ManagerID, Worker.EmployeeID, Lvl = 0
FROM EmployeeAdj AS Worker
WHERE Worker.EmployeeID = @orig_worker
UNION ALL
SELECT Worker.ManagerID, Worker.EmployeeID, Lvl = Lvl + 1
FROM EmployeeAdj AS Worker 
INNER JOIN
Bosses AS S ON S.Boss = Worker.EmployeeID
) 
SELECT Boss, @orig_worker AS Worker, Lvl FROM Bosses
WHERE Boss IS NOT NULL
GO

-- No intrinsic order, so I'll choose a column (EmployeeID) to order by
-- This won't necessarily get the same answer as the others, which are order-sensitive
-- But it will be the same in this case, because the order of the other tables
--  was derived from the EmployeeID of the original adjacency table
-- Oldest subordinate
DECLARE @my_member INT = 109
SELECT TOP(1) EmployeeID, 'is the most senior subordinate of', @my_member 
FROM EmployeeAdj
WHERE ManagerID = 109
ORDER BY EmployeeID
GO

-- Youngest subordinate
DECLARE @my_member INT = 109
SELECT TOP(1) EmployeeID, 'is the most senior subordinate of', @my_member 
FROM EmployeeAdj
WHERE ManagerID = 109
ORDER BY EmployeeID DESC
GO

-- nth sibling
DECLARE @n INT = 2
DECLARE @my_member INT = 109
SELECT EmployeeID, 'is the', @n, '-th subordinate of', @my_member
FROM
(
SELECT EmployeeID, ROW_NUMBER() OVER (ORDER BY EmployeeID) AS Num 
FROM EmployeeAdj
WHERE ManagerID = 109
) AS D
WHERE Num = @n

-- complete list of siblings
SELECT ManagerID, EmployeeID, ROW_NUMBER() OVER (PARTITION BY ManagerID ORDER BY EmployeeID) as Num
FROM EmployeeAdj
WHERE ManagerID IS NOT NULL