
USE MASTER
GO

IF DATABASEPROPERTYEX('Trees', 'Version') > 0
DROP DATABASE Trees
GO

CREATE DATABASE Trees
GO

USE Trees
GO

-- adjacency model
--

IF OBJECTPROPERTY(Object_Id('dbo.Personnel'), 'IsTable') = 1
DROP TABLE dbo.Personnel
GO

CREATE TABLE dbo.Personnel (
 emp VARCHAR(20) PRIMARY KEY, -- node
 boss VARCHAR(20) NULL,       -- edge
 salary DECIMAL(6,2) NOT NULL
)
GO

INSERT INTO Personnel VALUES('Albert', NULL, 1000.00);
INSERT INTO Personnel VALUES('Bert', 'Albert', 900.00);
INSERT INTO Personnel VALUES('Chuck', 'Albert', 900.00);
INSERT INTO Personnel VALUES('Donna', 'Chuck', 800.00);
INSERT INTO Personnel VALUES('Eddie', 'Chuck', 700.00);
INSERT INTO Personnel VALUES('Fred', 'Chuck', 600.00);
GO

-- find leaves
SELECT * 
 FROM Personnel AS E1
 WHERE NOT EXISTS(SELECT * FROM Personnel AS E2
                    WHERE E1.emp = E2.boss)
GO

-- find levels - one level
SELECT B1.emp, 'bosses', E1.emp
 FROM Personnel AS B1
 JOIN Personnel AS E1 ON B1.emp = E1.boss
GO

-- find level - second levels
SELECT B1.emp, 'bosses', E2.emp
 FROM Personnel AS B1
 JOIN Personnel AS E1 ON B1.emp = E1.boss
 JOIN Personnel AS E2 ON E1.emp = E2.boss
GO

IF OBJECTPROPERTY(Object_Id('dbo.TransClosure'), 'IsTable') = 1
DROP TABLE dbo.TransClosure
GO

-- Transitive Closure Table
CREATE TABLE dbo.TransClosure (
  start VARCHAR(20) NOT NULL
    REFERENCES Personnel(emp),
  finish VARCHAR(20) NOT NULL
    REFERENCES Personnel(emp),
  pathlength INTEGER NOT NULL DEFAULT 1
    CONSTRAINT positive_pathlength CHECK(pathlength > 0),
  CONSTRAINT no_cycles CHECK (start <> finish),
  PRIMARY KEY (start, finish)
)
GO
 
BEGIN 
DECLARE @old_size INTEGER
DECLARE @new_size INTEGER

INSERT INTO TransClosure (start, finish, pathlength)
  SELECT B1.emp, E1.emp, 1
    FROM Personnel AS B1
    JOIN Personnel AS E1 ON B1.emp = E1.boss

SET @old_size = (SELECT COUNT(*) FROM TransClosure);
SET @new_size = @old_size + 1;

WHILE (@old_size < @new_size)
BEGIN
INSERT INTO TransClosure (start, finish, pathlength)
  SELECT DISTINCT P1.start, P2.finish, (P1.pathlength + 1)
    FROM TransClosure AS P1
    JOIN TransClosure AS P2 ON P1.finish = P2.start
  WHERE P2.pathlength = 1
  AND   P1.pathlength = (SELECT MAX(pathlength) FROM TransClosure);  
  
  SET @old_size = @new_size;
  SET @new_size = (SELECT COUNT(*) FROM TransClosure);
END 
END
GO

SELECT * FROM TransClosure
GO

-- These don't touch the base table though

-- delete leaf
DELETE FROM TransClosure
 WHERE finish = 'Eddie'

-- delete subtree
DELETE FROM TransClosure
 WHERE start = 'Albert'

-- nested set

DROP TABLE dbo.NestedSet
GO

CREATE TABLE dbo.NestedSet (
  emp VARCHAR(20) PRIMARY KEY,
  lft INT, -- left
  rgt INT  -- right
)
GO

INSERT NestedSet VALUES ('Alice', 1, 28)
INSERT NestedSet VALUES ('Bob', 2, 9)
INSERT NestedSet VALUES ('B1', 3, 4)
INSERT NestedSet VALUES ('B2', 5, 6)
INSERT NestedSet VALUES ('B3', 7, 8)
INSERT NestedSet VALUES ('Carol', 10, 15)
INSERT NestedSet VALUES ('C1', 11, 12)
INSERT NestedSet VALUES ('C2', 13, 14)
INSERT NestedSet VALUES ('Dave', 16, 27)
INSERT NestedSet VALUES ('Eddie', 17, 20)
INSERT NestedSet VALUES ('E1', 18, 19)
INSERT NestedSet VALUES ('Frank', 21, 26)
INSERT NestedSet VALUES ('F1', 22, 23)
INSERT NestedSet VALUES ('F2', 24, 25)
GO

CREATE INDEX I1 ON NestedSet(lft, rgt)
GO

-- head of company
SELECT * FROM NestedSet WHERE lft = 1

-- leaf nodes
SELECT * FROM NestedSet WHERE lft = (rgt - 1)

-- subodinates
SELECT M.emp AS Manager, E.emp AS Employee
FROM   NestedSet AS M, NestedSet AS E
WHERE  E.lft BETWEEN M.lft AND M.rgt
AND E.lft <> M.lft
ORDER BY M.Emp


